from CvPythonExtensions import *

gc = CyGlobalContext()
translator = CyTranslator()
interface = CyInterface()


class ColAlerts:
	def __init__(self, eventManager):
		SettlementGrowth(eventManager)
		SettlementSupply(eventManager)
		SettlementStorage(eventManager)
		ImmigrantReady(eventManager)
		#FoundingFathers(eventManager)


class SettlementGrowth:
	"""Displays an alert when a settlement will grow or shrink on the next
	turn.
	
	This is a pending counterpart to the existing growth/starvation alerts.
	It accounts for net production/consumption within the settlement, but not
	food that will arrive next turn from trade routes.
	
	"""
	def __init__(self, eventManager):
		eventManager.addEventHandler("endTurnReady", self._onEndTurnReady)

	def _onEndTurnReady(self, argsList):
		player = gc.getActivePlayer()
		for cityID in range(player.getNumCities()):
			city = player.getCity(cityID)
			change = self._checkCity(city)
			if change == True:
				messageKey = "TXT_KEY_COLALERTS_ON_GROWTH_PENDING"
				interface.addMessage(
					player.getID(),
					True,
					gc.getDefineINT("EVENT_MESSAGE_TIME"),
					translator.getText(messageKey, (city.getName(),)),
					"AS2D_POSITIVE_DINK",
					InterfaceMessageTypes.MESSAGE_TYPE_INFO,
					gc.getYieldInfo(YieldTypes.YIELD_FOOD).getButton(),
					gc.getInfoTypeForString("COLOR_GREEN"),
					city.getX(),
					city.getY(),
					True,
					True)
			elif change == False:
				messageKey = "TXT_KEY_COLALERTS_ON_STARVATION_PENDING"
				interface.addMessage(
					player.getID(),
					True,
					gc.getDefineINT("EVENT_MESSAGE_TIME"),
					translator.getText(messageKey, (city.getName(),)),
					"AS2D_DEAL_CANCELLED",
					InterfaceMessageTypes.MESSAGE_TYPE_INFO,
					gc.getYieldInfo(YieldTypes.YIELD_FOOD).getButton(),
					gc.getInfoTypeForString("COLOR_RED"),
					city.getX(),
					city.getY(),
					True,
					True)

	def _checkCity(self, city):
		"""Returns True if the specified city will grow, False if it will
		shrink, or None if there will be no change.

		"""
		difference = city.foodDifference()
		turnsLeft = city.getFoodTurnsLeft()
		if difference > 0 and turnsLeft == 1 and not city.AI_avoidGrowth():
			return True
		if difference < 0 and city.getFood() // -difference == 0:
			return False


class SettlementSupply:
	"""Displays an alert when a settlement will lose or has lost yield output
	because a building does not have enough input yield to work at full
	capacity.

	Accounts for production/consumption within the settlement, but not raw
	goods that will arrive next turn from trade routes.

	"""
	def __init__(self, eventManager):
		eventManager.addEventHandler("GameStart", self._init)
		eventManager.addEventHandler("OnLoad", self._init)
		eventManager.addEventHandler("endTurnReady", self._onEndTurnReady)
		eventManager.addEventHandler("BeginPlayerTurn", self._onBeginPlayerTurn)
		self.yields = {}

	def _init(self, argsList):
		"""Builds a table of corresponding raw and finished yields.
		
		For now this assumes one output per input as in vanilla Colonization,
		but that's still more flexible than building the table by hand.
		
		"""
		for professionID in range(gc.getNumProfessionInfos()):
			professionInfo = gc.getProfessionInfo(professionID)
			consumed = professionInfo.getYieldConsumed()
			produced = professionInfo.getYieldProduced()
			if consumed != YieldTypes.NO_YIELD and produced != YieldTypes.NO_YIELD:
				self.yields[produced] = consumed

	def _onEndTurnReady(self, argsList):
		self._checkCities(True)

	def _onBeginPlayerTurn(self, argsList):
		gameTurn, playerID = argsList
		if playerID == gc.getActivePlayer().getID():
			self._checkCities(False)

	def _checkCities(self, pending):
		player = gc.getActivePlayer()
		for cityID in range(player.getNumCities()):
			city = player.getCity(cityID)
			for outputYield, inputYield in self.yields.iteritems():
				outputLost = self._checkYield(city, outputYield, inputYield)
				if outputLost > 0:
					outputInfo = gc.getYieldInfo(outputYield)
					messageKey = "TXT_KEY_COLALERTS_ON_LIMITED_SUPPLY"
					if pending:
						messageKey += "_PENDING"
					replacements = outputLost, outputInfo.getChar(), city.getName()
					interface.addMessage(
						player.getID(),
						True,
						gc.getDefineINT("EVENT_MESSAGE_TIME"),
						translator.getText(messageKey, replacements),
						"AS2D_DEAL_CANCELLED",
						InterfaceMessageTypes.MESSAGE_TYPE_MINOR_EVENT,
						None,
						gc.getInfoTypeForString("COLOR_RED"),
						city.getX(),
						city.getY(),
						True,
						True)

	def _checkYield(self, city, outputYield, inputYield):
		"""Returns the amount of the specified output yield that will be "lost"
		in the specified settlement due to limited input yield.

		"""
		rawOutput = city.getRawYieldProduced(outputYield)
		if rawOutput > 0:
			actualOutput = city.calculateActualYieldProduced(outputYield)
			return rawOutput - actualOutput


class SettlementStorage:
	"""Displays an alert when a settlement will lose or sell off excess goods
	due to limited storage.

	This is a pending counterpart to the existing storage alert.

	"""
	def __init__(self, eventManager):
		eventManager.addEventHandler("endTurnReady", self._onEndTurnReady)

	def _onEndTurnReady(self, argsList):
		player = gc.getActivePlayer()
		for cityID in range(player.getNumCities()):
			city = player.getCity(cityID)
			for yieldType in range(YieldTypes.NUM_YIELD_TYPES):
				if yieldType == YieldTypes.YIELD_FOOD:
					continue
				lost, profit = self._checkYield(player, city, yieldType)
				if lost > 0:
					messageKey = "TXT_KEY_COLALERTS_ON_LIMITED_STORAGE"
					if profit > 0:
						messageKey += "_SOLD"
					yieldInfo = gc.getYieldInfo(yieldType)
					replacements = lost, yieldInfo.getChar(), city.getName(), profit
					interface.addMessage(
						player.getID(),
						True,
						gc.getDefineINT("EVENT_MESSAGE_TIME"),
						translator.getText(messageKey, replacements),
						"AS2D_DEAL_CANCELLED",
						InterfaceMessageTypes.MESSAGE_TYPE_MINOR_EVENT,
						None,
						gc.getInfoTypeForString("COLOR_RED"),
						city.getX(),
						city.getY(),
						True,
						True)

	def _checkYield(self, player, city, yieldType):
		"""Returns the amount of the specified yield that will be discarded or
		sold in the specified city, as well as the sale profit if any.

		If a settlement ends up with more to store than it has room for, only a
		certain percentage of the total will decay per turn, with a minimum
		amount decayed if excess is at or over that amount. By default this is
		10% and 5 units respectively.

		"""
		produced = city.calculateActualYieldProduced(yieldType)
		stored = city.getYieldStored(yieldType)
		capacity = city.getMaxYieldCapacity()
		excess = produced + stored - capacity
		if excess > 0:
			decayPercent = gc.getDefineINT("CITY_YIELD_DECAY_PERCENT")
			minDecay = gc.getDefineINT("MIN_CITY_YIELD_DECAY")
			actualDecay = max(decayPercent * excess / 100, minDecay)
			yieldLost = min(excess, actualDecay)
			percentSold = self._getOverflowYieldSellPercent(city)
			totalProfit = player.getSellToEuropeProfit(yieldType, yieldLost)
			actualProfit = percentSold * totalProfit / 100
			return yieldLost, actualProfit
		else:
			return 0, 0

	def _getOverflowYieldSellPercent(self, city):
		"""Returns the percentage of excess yields that will be sold in the
		specified city rather than discarded, based on which buildings it has.

		By default this is 50% with an Expanded Warehouse. This function is
		redifined here because the CvCity function isn't exposed to Python.

		"""
		sellPercent = 0
		for buildingClass in range(gc.getNumBuildingClassInfos()):
			civilization = gc.getCivilizationInfo(city.getCivilizationType())
			buildingID = civilization.getCivilizationBuildings(buildingClass)
			if buildingID and buildingID != BuildingTypes.NO_BUILDING and \
					city.isHasBuilding(buildingID):
				buildingInfo = gc.getBuildingInfo(buildingID)
				sellPercent = max(sellPercent, buildingInfo.getOverflowSellPercent())
		return sellPercent


class ImmigrantReady:
	"""Displays an alert if a European immigrant will be available on the next
	turn.

	This is a pending counterpart to the existing immigration alert.

	"""
	def __init__(self, eventManager):
		eventManager.addEventHandler("endTurnReady", self._onEndTurnReady)

	def _onEndTurnReady(self, argsList):
		player = gc.getActivePlayer()
		if self._isImmigrantPending(player):
			messageKey = "TXT_KEY_COLALERTS_ON_PENDING_IMMIGRANT"
			interface.addMessage(
				player.getID(),
				True,
				gc.getDefineINT("EVENT_MESSAGE_TIME"),
				translator.getText(messageKey, ()),
				"AS2D_UNIT_GREATPEOPLE",
				InterfaceMessageTypes.MESSAGE_TYPE_INFO,
				None,
				gc.getInfoTypeForString("COLOR_UNIT_TEXT"),
				0,
				0,
				True,
				True)

	def _isImmigrantPending(self, player):
		rate = player.getYieldRate(YieldTypes.YIELD_CROSSES)
		stored = player.getCrossesStored()
		threshold = player.immigrationThreshold()
		return rate + stored >= threshold


class FoundingFathers:
	def __init__(self, eventManager):
		pass
